@@ -64,10 +64,12 @@ from gettext import gettext as _
 
 from Common.utils import *
 from Common.SimpleGladeApp import SimpleGladeApp
-from DistUpgradeFetcher import DistUpgradeFetcher
+from DistUpgradeFetcher import DistUpgradeFetcherGtk
+from ChangelogViewer import ChangelogViewer
 import GtkProgress
 
-from MetaRelease import Dist, MetaRelease
+from Core.MetaRelease import Dist
+from MetaReleaseGObject import MetaRelease
 
 #import pdb
 
@@ -188,15 +190,16 @@ class MyCache(apt.Cache):
                     break
                 match = re.match(regexp,line)
                 if match:
-                    # FIXME: the installed version can have a epoch, but th
-                    #        changelog does not have one, we do a dumb
-                    #        approach here and just strip it away, but I'm
-                    #        sure that this can lead to problems
+                    # strip epoch from installed version
+                    # and from changelog too
                     installed = pkg.installedVersion
                     if installed and ":" in installed:
                         installed = installed.split(":",1)[1]
+                    changelogver = match.group(1)
+                    if changelogver and ":" in changelogver:
+                        changelogver = changelogver.split(":",1)[1]
                     if installed and \
-                        apt_pkg.VersionCompare(match.group(1),installed)<=0:
+                        apt_pkg.VersionCompare(changelogver,installed)<=0:
                         break
                 # EOF (shouldn't really happen)
                 alllines = alllines + line
@@ -233,7 +236,8 @@ class UpdateList:
     pipe = os.popen("lsb_release -c -s")
     dist = pipe.read().strip()
     del pipe
-
+    self.distUpgradeWouldDelete = 0
+    
     templates = [("%s-security" % dist, "Ubuntu", _("Important security updates")
                                                     , 10),
                  ("%s-updates" % dist, "Ubuntu", _("Recommended updates"), 9),
@@ -312,6 +316,9 @@ class UpdateManager(SimpleGladeApp):
     self.dl_size = 0
 
     # create text view
+    self.textview_changes = ChangelogViewer()
+    self.textview_changes.show()
+    self.scrolledwindow_changes.add(self.textview_changes)
     changes_buffer = self.textview_changes.get_buffer()
     changes_buffer.create_tag("versiontag", weight=pango.WEIGHT_BOLD)
 
@@ -394,7 +401,7 @@ class UpdateManager(SimpleGladeApp):
           if use_auth:
               auth_user = self.gconfclient.get_string("/system/http_proxy/authentication_user")
               auth_pw = self.gconfclient.get_string("/system/http_proxy/authentication_password")
-              proxy = "http://%s:%s@%s:%s/" % (auth_user,auth_pass,host, port)
+              proxy = "http://%s:%s@%s:%s/" % (auth_user,auth_pw,host, port)
           else:
               proxy = "http://%s:%s/" % (host, port)
       if proxy:
@@ -436,15 +443,15 @@ class UpdateManager(SimpleGladeApp):
     except:
         print "warning: could not initiate dbus"
         return
-    proxy_obj = bus.get_object('org.freedesktop.UpdateManager', 
-                               '/org/freedesktop/UpdateManagerObject')
-    iface = dbus.Interface(proxy_obj, 'org.freedesktop.UpdateManagerIFace')
     try:
+        proxy_obj = bus.get_object('org.freedesktop.UpdateManager', 
+                                   '/org/freedesktop/UpdateManagerObject')
+        iface = dbus.Interface(proxy_obj, 'org.freedesktop.UpdateManagerIFace')
         iface.bringToFront()
         #print "send bringToFront"
         sys.exit(0)
     except dbus.DBusException, e:
-         print "no listening object (%s) "% e
+         #print "no listening object (%s) "% e
          bus_name = dbus.service.BusName('org.freedesktop.UpdateManager',bus)
          self.dbusControler = UpdateManagerDbusControler(self, bus_name)
 
@@ -583,6 +590,7 @@ class UpdateManager(SimpleGladeApp):
     self.setBusy(True)
     self.cache.saveDistUpgrade()
     self.treeview_update.queue_draw()
+    self.refresh_updates_count()
     self.setBusy(False)
 
   def select_none_updgrades(self, widget):
@@ -592,6 +600,7 @@ class UpdateManager(SimpleGladeApp):
     self.setBusy(True)
     self.cache.clear()
     self.treeview_update.queue_draw()
+    self.refresh_updates_count()
     self.setBusy(False)
 
   def setBusy(self, flag):
@@ -608,10 +617,14 @@ class UpdateManager(SimpleGladeApp):
 
   def refresh_updates_count(self):
       self.button_install.set_sensitive(self.cache.installCount)
-      self.dl_size = self.cache.requiredDownload
-      # TRANSLATORS: b stands for Bytes
-      self.label_downsize.set_markup(_("Download size: %s") % \
-                                       humanize_size(self.dl_size))
+      try:
+          self.dl_size = self.cache.requiredDownload
+          # TRANSLATORS: b stands for Bytes
+          self.label_downsize.set_markup(_("Download size: %s") % \
+                                         humanize_size(self.dl_size))
+      except SystemError, e:
+          print "requiredDownload could not be calculated: %s" % e
+          self.label_downsize.set_markup(_("Unknown download size"))
       
   def update_count(self):
       """activate or disable widgets and show dialog texts correspoding to
@@ -693,7 +706,7 @@ class UpdateManager(SimpleGladeApp):
   def on_button_install_clicked(self, widget):
     #print "on_button_install_clicked"
     self.invoke_manager(INSTALL)
-
+    
   def invoke_manager(self, action):
     # check first if no other package manager is runing
 
@@ -743,7 +756,10 @@ class UpdateManager(SimpleGladeApp):
   def allow_sleep(self, dev, cookie):
     """Send a dbus signal to gnome-power-manager to allow a suspending
     the system"""
-    dev.UnInhibit(cookie)
+    try:
+      dev.UnInhibit(cookie)
+    except Exception, e:
+      print "could not send the dbus UnInhibit signal: %s" % e
 
   def toggled(self, renderer, path):
     """ a toggle button in the listview was toggled """
@@ -751,7 +767,7 @@ class UpdateManager(SimpleGladeApp):
     pkg = self.store.get_value(iter, LIST_PKG)
     # make sure that we don't allow to toggle deactivated updates
     # this is needed for the call by the row activation callback
-    if pkg.name in self.list.held_back:
+    if pkg is None or pkg.name in self.list.held_back:
         return False
     self.setBusy(True)
     # update the cache
@@ -776,7 +792,7 @@ class UpdateManager(SimpleGladeApp):
   def exit(self):
     """ exit the application, save the state """
     self.save_state()
-    gtk.main_quit()
+    #gtk.main_quit()
     sys.exit(0)
 
   def save_state(self):
@@ -802,15 +818,48 @@ class UpdateManager(SimpleGladeApp):
   def fillstore(self):
     # use the watch cursor
     self.setBusy(True)
-
     # clean most objects
     self.dl_size = 0
-    self.initCache()
+    try:
+        self.initCache()
+    except SystemError, e:
+        msg = ("<big><b>%s</b></big>\n\n%s\n'%s'" %
+               (_("Could not initialize the package information"),
+                _("A unresolvable problem occurred while "
+                  "initializing the package information.\n\n"
+                  "Please report this bug against the 'update-manager' "
+                  "package and include the following error message:\n"),
+                e)
+               )
+        dialog = gtk.MessageDialog(self.window_main,
+                                   0, gtk.MESSAGE_ERROR,
+                                   gtk.BUTTONS_CLOSE,"")
+        dialog.set_markup(msg)
+        dialog.vbox.set_spacing(6)
+        dialog.run()
+        dialog.destroy()
+        sys.exit(1)
     self.store.clear()
     self.list = UpdateList()
-
     # fill them again
-    self.list.update(self.cache)
+    try:
+        self.list.update(self.cache)
+    except SystemError, e:
+        msg = ("<big><b>%s</b></big>\n\n%s\n'%s'" %
+               (_("Could not calculate the upgrade"),
+                _("A unresolvable problem occurred while "
+                  "calculating the upgrade.\n\n"
+                  "Please report this bug against the 'update-manager' "
+                  "package and include the following error message:"),
+                e)
+               )
+        dialog = gtk.MessageDialog(self.window_main,
+                                   0, gtk.MESSAGE_ERROR,
+                                   gtk.BUTTONS_CLOSE,"")
+        dialog.set_markup(msg)
+        dialog.vbox.set_spacing(6)
+        dialog.run()
+        dialog.destroy()
     if self.list.num_updates > 0:
       origin_list = self.list.pkgs.keys()
       origin_list.sort(lambda x,y: cmp(x.importance,y.importance))
@@ -855,7 +904,13 @@ class UpdateManager(SimpleGladeApp):
 
   def on_button_dist_upgrade_clicked(self, button):
       #print "on_button_dist_upgrade_clicked"
-      fetcher = DistUpgradeFetcher(self, self.new_dist)
+      progress = GtkProgress.GtkFetchProgress(self,
+                                              _("Downloading the upgrade "
+                                                "tool"),
+                                              _("The upgrade tool will "
+                                                "guide you through the "
+                                                "upgrade process."))
+      fetcher = DistUpgradeFetcherGtk(new_dist=self.new_dist, parent=self, progress=progress)
       fetcher.run()
       
   def new_dist_available(self, meta_release, upgradable_to):
@@ -945,7 +1000,7 @@ class UpdateManager(SimpleGladeApp):
 
   def main(self, options):
     gconfclient = gconf.client_get_default() 
-    self.meta = MetaRelease(options.devel_release)
+    self.meta = MetaRelease(options.devel_release, options.use_proposed)
     self.meta.connect("dist_no_longer_supported",self.dist_no_longer_supported)
 
     # check if we are interessted in dist-upgrade information
